import ipaddress
import markdown as md_package
import re
from flask import Blueprint

bp = Blueprint("filters", __name__)


def github_style_admonition(text):
    """Transform GitHub-style alerts (> [!NOTE], etc.) to custom HTML with GitHub styling"""
    # Process the text line by line for better control
    lines = text.split('\n')
    result_lines = []
    i = 0
    
    while i < len(lines):
        line = lines[i]
        
        # Check if this line starts a GitHub-style alert
        alert_match = re.match(r'>\s*\[!(NOTE|TIP|IMPORTANT|WARNING|CAUTION)\]\s*(.*)', line)
        
        if alert_match:
            # Found an alert start
            alert_type = alert_match.group(1).lower()
            first_line_content = alert_match.group(2).strip()
            
            # Collect all continuation lines (those starting with >)
            alert_content = [first_line_content] if first_line_content else []
            j = i + 1
            
            while j < len(lines) and lines[j].strip().startswith('>'):
                # Remove the > and one optional space after it
                content = re.sub(r'^>\s?', '', lines[j].strip())
                alert_content.append(content)
                j += 1
            
            # Create the HTML for the alert with proper styling
            alert_html = f'<div class="markdown-alert markdown-alert-{alert_type}">\n'
            alert_html += f'<p class="markdown-alert-title">{alert_type.capitalize()}</p>\n'
            
            # Join the content with line breaks
            if alert_content:
                alert_html += '\n'.join(alert_content)
            
            alert_html += '\n</div>'
            
            # Add the processed alert HTML
            result_lines.append(alert_html)
            
            # Move past all the lines we've processed
            i = j
        else:
            # Not an alert line, add it unchanged
            result_lines.append(line)
            i += 1
    
    return '\n'.join(result_lines)


# Register template filters
@bp.app_template_filter('github_style_admonition')
def github_style_admonition_filter(text):
    """Template filter version of github_style_admonition"""
    if text is None:
        return ""
    return github_style_admonition(text)


@bp.app_template_filter("markdown")
def markdown_filter(text):
    """Convert markdown text to HTML with proper GitHub styling for all elements"""
    if not text:
        return ""
    
    # First process GitHub-style alerts
    processed_text = github_style_admonition(text)
    
    # Convert to HTML with standard markdown
    html = md_package.markdown(
        processed_text, 
        extensions=["tables", "fenced_code", "codehilite", "nl2br", "sane_lists", "smarty"]
    )
    
    # Wrap in GitHub's markdown reading view class for consistent styling
    html = f'<div class="markdown-reading-view">{html}</div>'
    
    return html


@bp.app_template_filter("ip_network")
def ip_network_filter(cidr):
    """Convert a CIDR string to an IP network object"""
    try:
        return ipaddress.ip_network(cidr, strict=False)
    except ValueError:
        return None


@bp.app_template_filter("ip_address")
def ip_address_filter(ip):
    """Convert an IP string to an IP address object"""
    try:
        return ipaddress.ip_address(ip)
    except ValueError:
        return None


@bp.app_template_global("get_ip_network")
def get_ip_network(cidr):
    """Global function to get an IP network object from CIDR"""
    try:
        return ipaddress.ip_network(cidr, strict=False)
    except ValueError:
        return None
